@@ -7,10 +7,6 @@ separate_testing do
 end
 
 describe Rack::Response do
-  deprecated "#header returns headers" do
-    Rack::Response[200, { "v" => "1" }, []].header['v'].must_equal '1'
-  end
-
   it 'has standard constructor' do
     headers = { "header" => "value" }
     body = ["body"]
@@ -22,6 +18,12 @@ describe Rack::Response do
     response.body.must_equal body
   end
 
+  it 'raises ArgumentError unless headers is a hash' do
+    lambda {
+      Rack::Response.new(nil, 200, Object.new)
+    }.must_raise(ArgumentError)
+  end
+
   it 'has cache-control methods' do
     response = Rack::Response.new
     cc = 'foo'
@@ -64,7 +66,7 @@ describe Rack::Response do
     }
   end
 
-  it "can be written to inside finish block, but does not update content-length" do
+  it "can be written to inside finish block and it does not generate a content-length header" do
     response = Rack::Response.new('foo')
     response.write "bar"
 
@@ -76,7 +78,7 @@ describe Rack::Response do
     body.each { |part| parts << part }
 
     parts.must_equal ["foo", "bar", "baz"]
-    h['content-length'].must_equal '6'
+    h['content-length'].must_be_nil
   end
 
   it "#write calls #<< on non-iterable body" do
@@ -168,82 +170,82 @@ describe Rack::Response do
     response["set-cookie"].must_equal "foo=bar"
   end
 
-  it "can set SameSite cookies with symbol value :none" do
+  it "can set same site cookies with symbol value :none" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: :none }
-    response["set-cookie"].must_equal "foo=bar; SameSite=None"
+    response["set-cookie"].must_equal "foo=bar; samesite=none"
   end
 
-  it "can set SameSite cookies with symbol value :None" do
+  it "can set same site cookies with symbol value :None" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: :None }
-    response["set-cookie"].must_equal "foo=bar; SameSite=None"
+    response["set-cookie"].must_equal "foo=bar; samesite=none"
   end
 
-  it "can set SameSite cookies with string value 'None'" do
+  it "can set same site cookies with string value 'None'" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: "None" }
-    response["set-cookie"].must_equal "foo=bar; SameSite=None"
+    response["set-cookie"].must_equal "foo=bar; samesite=none"
   end
 
-  it "can set SameSite cookies with symbol value :lax" do
+  it "can set same site cookies with symbol value :lax" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: :lax }
-    response["set-cookie"].must_equal "foo=bar; SameSite=Lax"
+    response["set-cookie"].must_equal "foo=bar; samesite=lax"
   end
 
-  it "can set SameSite cookies with symbol value :Lax" do
+  it "can set same site cookies with symbol value :Lax" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: :lax }
-    response["set-cookie"].must_equal "foo=bar; SameSite=Lax"
+    response["set-cookie"].must_equal "foo=bar; samesite=lax"
   end
 
-  it "can set SameSite cookies with string value 'Lax'" do
+  it "can set same site cookies with string value 'Lax'" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: "Lax" }
-    response["set-cookie"].must_equal "foo=bar; SameSite=Lax"
+    response["set-cookie"].must_equal "foo=bar; samesite=lax"
   end
 
-  it "can set SameSite cookies with boolean value true" do
+  it "can set same site cookies with boolean value true" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: true }
-    response["set-cookie"].must_equal "foo=bar; SameSite=Strict"
+    response["set-cookie"].must_equal "foo=bar; samesite=strict"
   end
 
-  it "can set SameSite cookies with symbol value :strict" do
+  it "can set same site cookies with symbol value :strict" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: :strict }
-    response["set-cookie"].must_equal "foo=bar; SameSite=Strict"
+    response["set-cookie"].must_equal "foo=bar; samesite=strict"
   end
 
-  it "can set SameSite cookies with symbol value :Strict" do
+  it "can set same site cookies with symbol value :Strict" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: :Strict }
-    response["set-cookie"].must_equal "foo=bar; SameSite=Strict"
+    response["set-cookie"].must_equal "foo=bar; samesite=strict"
   end
 
-  it "can set SameSite cookies with string value 'Strict'" do
+  it "can set same site cookies with string value 'Strict'" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: "Strict" }
-    response["set-cookie"].must_equal "foo=bar; SameSite=Strict"
+    response["set-cookie"].must_equal "foo=bar; samesite=strict"
   end
 
-  it "validates the SameSite option value" do
+  it "validates the same site option value" do
     response = Rack::Response.new
     lambda {
       response.set_cookie "foo", { value: "bar", same_site: "Foo" }
     }.must_raise(ArgumentError).
-      message.must_match(/Invalid SameSite value: "Foo"/)
+      message.must_match(/Invalid :same_site value: "Foo"/)
   end
 
-  it "can set SameSite cookies with symbol value" do
+  it "can set same site cookies with symbol value" do
     response = Rack::Response.new
     response.set_cookie "foo", { value: "bar", same_site: :Strict }
-    response["set-cookie"].must_equal "foo=bar; SameSite=Strict"
+    response["set-cookie"].must_equal "foo=bar; samesite=strict"
   end
 
   [ nil, false ].each do |non_truthy|
-    it "omits SameSite attribute given a #{non_truthy.inspect} value" do
+    it "omits same site attribute given a #{non_truthy.inspect} value" do
       response = Rack::Response.new
       response.set_cookie "foo", { value: "bar", same_site: non_truthy }
       response["set-cookie"].must_equal "foo=bar"
@@ -592,16 +594,19 @@ describe Rack::Response do
     res.status = 200
     res.headers["content-length"] = "10"
     res.finish
+    # We don't overwrite the content-length if it's already set - e.g. HEAD response may not have a body...
     res.headers["content-length"].must_equal "10"
   end
 
-  it "updates content-length when body appended to using #write" do
+  it "updates length when body appended to using #write" do
     res = Rack::Response.new
     res.status = 200
-    res.headers["content-length"].must_be_nil
+    res.length.must_be_nil
     res.write "Hi"
-    res.headers["content-length"].must_equal "2"
+    res.length.must_equal 2
     res.write " there"
+    res.length.must_equal 8
+    res.finish
     res.headers["content-length"].must_equal "8"
   end
 
@@ -627,6 +632,26 @@ describe Rack::Response do
     res.finish.last.must_equal ["Foo", "Bar"]
   end
 
+  it "handles string reuse in existing body when calling #write" do
+    body_class = Class.new do
+      def initialize(file)
+        @file = file
+      end
+
+      def each
+        buffer = String.new
+
+        while @file.read(5, buffer)
+          yield(buffer)
+        end
+      end
+    end
+    body = body_class.new(StringIO.new('Large large file content'))
+    res = Rack::Response.new(body)
+    res.write(" written")
+    res.finish.last.must_equal ["Large", " larg", "e fil", "e con", "tent", " written"]
+  end
+
   it "calls close on #body" do
     res = Rack::Response.new
     res.body = StringIO.new
